# frozen_string_literal: true

require_relative 'environment_builder'
require_relative 'lambda_context'

module Rack
  module Shelf
    # Converts AWS API Gateway events given to Lambda
    # to Rack environment instances.
    class APIGateway
      include LambdaContext

      # Produces a Rack compatible environment instance
      # from a Lambda event and context.
      # @param event [Hash] Event from the Lambda handler.
      # @param context [Object] Context from the Lambda handler.
      # @return [Hash] Rack environment.
      def self.env(event, context)
        new(event, context).build
      end

      # Creates an instance dedicated to building a Rack environment
      # for a single event and context.
      # @param event [Hash] Event from the Lambda handler.
      # @param context [Object] Context from the Lambda handler.
      def initialize(event, context)
        @event   = event
        @context = context
        @builder = EnvironmentBuilder.new
      end

      # Builds the rack environment.
      # @return [Hash] Rack environment.
      def build
        build_common
        build_headers
        build_body
        build_lambda_context

        builder.build
      end

      private

      # Rack environment builder instance.
      # @return [EnvironmentBuilder]
      attr_reader :builder

      # Event from the Lambda handler.
      # @return [Hash]
      attr_reader :event

      # Context from the Lambda handler.
      # @return [Object]
      attr_reader :context

      # Retrieves the HTTP headers from the Lambda event.
      # @return [Hash] HTTP headers.
      def headers
        event['headers'] || {}
      end

      # Retrieves the HTTP request method from the Lambda event.
      # @return [String] HTTP request method.
      def request_method
        event.fetch('httpMethod')
      end

      # Retrieves the path information, or a default value.
      # @return [String]
      def path_info
        event['path'] || '/'
      end

      # Retrieves the query parameters from the Lambda event.
      # @return [Hash]
      def query_params
        event['queryStringParameters'] || {}
      end

      # Retrieves the server hostname, or a default value.
      # @return [String]
      def server_name
        event['Host'] || 'localhost'
      end

      # Retrieves the server port, or a default value.
      # @return [Integer]
      def server_port
        event['X-Forwarded-Port'] || 80
      end

      # Retrieves the URL scheme, or a default value.
      # @return [String]
      def url_scheme
        headers.fetch('CloudFront-Forwarded-Proto') do
          headers.fetch('X-Forwarded-Proto', 'http')
        end
      end

      # Configures common Rack environment attributes.
      # @return [void]
      def build_common # rubocop:disable Metrics/AbcSize
        builder.request_method(request_method)
        builder.path_info(path_info)
        builder.query_params(query_params)
        builder.server_name(server_name)
        builder.server_port(server_port)
        builder.url_scheme(url_scheme)
      end

      # Configures the HTTP request headers.
      # @return [void]
      def build_headers
        headers.each do |key, value|
          builder.header(key, value)
        end
      end

      # Configures the client request body portion.
      # @return [void]
      def build_body
        return unless (body = event['body'])

        if event['isBase64Encoded']
          builder.base64_body(body)
        else
          builder.body(body)
        end
      end
    end
  end
end
